@@ -10,56 +10,23 @@ import sys
 import time
 import unittest
 import warnings
+import contextlib
 
-try:
-    from unittest import mock
-except ImportError:
-    import mock
+import portend
+import pytest
 
-import nose
-import six
+from cheroot.test import webtest
 
 import cherrypy
-from cherrypy._cpcompat import text_or_bytes, copyitems, HTTPSConnection, ntob
+from cherrypy._cpcompat import text_or_bytes, HTTPSConnection, ntob
 from cherrypy.lib import httputil
 from cherrypy.lib import gctools
-from cherrypy.lib.reprconf import unrepr
-from cherrypy.test import webtest
 
-_testconfig = None
 log = logging.getLogger(__name__)
 thisdir = os.path.abspath(os.path.dirname(__file__))
 serverpem = os.path.join(os.getcwd(), thisdir, 'test.pem')
 
 
-def get_tst_config(overconf={}):
-    global _testconfig
-    if _testconfig is None:
-        conf = {
-            'scheme': 'http',
-            'protocol': 'HTTP/1.1',
-            'port': 54583,
-            'host': '127.0.0.1',
-            'validate': False,
-            'server': 'wsgi',
-        }
-        try:
-            import testconfig
-            _conf = testconfig.config.get('supervisor', None)
-            if _conf is not None:
-                for k, v in _conf.items():
-                    if isinstance(v, text_or_bytes):
-                        _conf[k] = unrepr(v)
-                conf.update(_conf)
-        except ImportError:
-            pass
-        _testconfig = conf
-    conf = _testconfig.copy()
-    conf.update(overconf)
-
-    return conf
-
-
 class Supervisor(object):
 
     """Base class for modeling and controlling servers during testing."""
@@ -71,7 +38,8 @@ class Supervisor(object):
             setattr(self, k, v)
 
 
-log_to_stderr = lambda msg, level: sys.stderr.write(msg + os.linesep)
+def log_to_stderr(msg, level):
+    return sys.stderr.write(msg + os.linesep)
 
 
 class LocalSupervisor(Supervisor):
@@ -125,7 +93,8 @@ class LocalSupervisor(Supervisor):
 
         cherrypy.engine.exit()
 
-        for name, server in copyitems(getattr(cherrypy, 'servers', {})):
+        servers_copy = list(getattr(cherrypy, 'servers', {}).items())
+        for name, server in servers_copy:
             server.unsubscribe()
             del cherrypy.servers[name]
 
@@ -274,7 +243,14 @@ class CPWebCase(webtest.WebCase):
     def setup_class(cls):
         ''
         # Creates a server
-        conf = get_tst_config()
+        conf = {
+            'scheme': 'http',
+            'protocol': 'HTTP/1.1',
+            'port': 54583,
+            'host': '127.0.0.1',
+            'validate': False,
+            'server': 'wsgi',
+        }
         supervisor_factory = cls.available_servers.get(
             conf.get('server', 'wsgi'))
         if supervisor_factory is None:
@@ -350,7 +326,7 @@ class CPWebCase(webtest.WebCase):
                                        protocol, raise_subcls)
 
     def skip(self, msg='skipped '):
-        raise nose.SkipTest(msg)
+        pytest.skip(msg)
 
     def assertErrorPage(self, status, message=None, pattern=''):
         """Compare the response body with a built in error page.
@@ -368,7 +344,7 @@ class CPWebCase(webtest.WebCase):
         epage = re.escape(page)
         epage = epage.replace(
             esc('<pre id="traceback"></pre>'),
-            esc('<pre id="traceback">') + ntob('(.*)') + esc('</pre>'))
+            esc('<pre id="traceback">') + b'(.*)' + esc('</pre>'))
         m = re.match(epage, self.body, re.DOTALL)
         if not m:
             self._handlewebError(
@@ -414,6 +390,8 @@ def _test_method_sorter(_, x, y):
     if x < y:
         return -1
     return 0
+
+
 unittest.TestLoader.sortTestMethodsUsing = _test_method_sorter
 
 
@@ -471,17 +449,36 @@ server.ssl_private_key: r'%s'
             'extra': extra,
         }
         with io.open(self.config_file, 'w', encoding='utf-8') as f:
-            f.write(six.text_type(conf))
+            f.write(str(conf))
 
     def start(self, imports=None):
         """Start cherryd in a subprocess."""
-        cherrypy._cpserver.wait_for_free_port(self.host, self.port)
+        portend.free(self.host, self.port, timeout=1)
 
         args = [
-            os.path.join(thisdir, '..', 'cherryd'),
+            '-m',
+            'cherrypy',
             '-c', self.config_file,
             '-p', self.pid_file,
         ]
+        r"""
+        Command for running cherryd server with autoreload enabled
+
+        Using
+
+        ```
+        ['-c',
+         "__requires__ = 'CherryPy'; \
+         import pkg_resources, re, sys; \
+         sys.argv[0] = re.sub(r'(-script\.pyw?|\.exe)?$', '', sys.argv[0]); \
+         sys.exit(\
+            pkg_resources.load_entry_point(\
+                'CherryPy', 'console_scripts', 'cherryd')())"]
+        ```
+
+        doesn't work as it's impossible to reconstruct the `-c`'s contents.
+        Ref: https://github.com/cherrypy/cherrypy/issues/1545
+        """
 
         if not isinstance(imports, (list, tuple)):
             imports = [imports]
@@ -506,7 +503,7 @@ server.ssl_private_key: r'%s'
         if self.wait:
             self.exit_code = self._proc.wait()
         else:
-            cherrypy._cpserver.wait_for_occupied_port(self.host, self.port)
+            portend.occupied(self.host, self.port, timeout=5)
 
         # Give the engine a wee bit more time to finish STARTING
         if self.daemonize:
@@ -526,20 +523,5 @@ server.ssl_private_key: r'%s'
         self._proc.wait()
 
     def _join_daemon(self):
-        try:
-            try:
-                # Mac, UNIX
-                os.wait()
-            except AttributeError:
-                # Windows
-                try:
-                    pid = self.get_pid()
-                except IOError:
-                    # Assume the subprocess deleted the pidfile on shutdown.
-                    pass
-                else:
-                    os.waitpid(pid, 0)
-        except OSError:
-            x = sys.exc_info()[1]
-            if x.args != (10, 'No child processes'):
-                raise
+        with contextlib.suppress(IOError):
+            os.waitpid(self.get_pid(), 0)
