@@ -6,10 +6,8 @@ import os
 import sys
 import types
 
-import six
-
 import cherrypy
-from cherrypy._cpcompat import ntob, ntou
+from cherrypy._cpcompat import ntou
 from cherrypy import _cptools, tools
 from cherrypy.lib import httputil, static
 
@@ -57,7 +55,7 @@ class CoreRequestHandlingTest(helper.CPWebCase):
             """
             def __init__(cls, name, bases, dct):
                 type.__init__(cls, name, bases, dct)
-                for value in six.itervalues(dct):
+                for value in dct.values():
                     if isinstance(value, types.FunctionType):
                         value.exposed = True
                 setattr(root, name.lower(), cls())
@@ -235,7 +233,7 @@ class CoreRequestHandlingTest(helper.CPWebCase):
                 return ['con', 'tent']
 
             def as_yield(self):
-                yield ntob('content')
+                yield b'content'
 
             @cherrypy.config(**{'tools.flatten.on': True})
             def as_dblyield(self):
@@ -279,8 +277,8 @@ class CoreRequestHandlingTest(helper.CPWebCase):
             def header_list(self):
                 pass
             header_list = cherrypy.tools.append_headers(header_list=[
-                (ntob('WWW-Authenticate'), ntob('Negotiate')),
-                (ntob('WWW-Authenticate'), ntob('Basic realm="foo"')),
+                (b'WWW-Authenticate', b'Negotiate'),
+                (b'WWW-Authenticate', b'Basic realm="foo"'),
             ])(header_list)
 
             def commas(self):
